#========================================================================================
# package      DE_EPAGES::Order::t::CartResource_Patch
# base         DE_EPAGES::Shop::API::TestImportXML
# state        public
#----------------------------------------------------------------------------------------
# description  Integration test for patch method in CartResource.
#========================================================================================
package DE_EPAGES::Order::t::CartResource_Patch;
use base DE_EPAGES::Shop::API::TestImportXML;

use DE_EPAGES::Order::API::Resource::Cart;
use Test::More;

use strict;



sub shopAlias {
    return 'RestApiTestShop';
}

sub importXMLPath {
    return 'ShopXMLs/Shop_for_CartResource_Patch_test.xml';
}

my $GuidOfAddedLineItem;




sub test {
    my $self = shift;
    $self->test_patch_modifyProductLineItem();
    $self->test_patch_addProductLineItem();
    $self->test_patch_removeProductLineItem();
    $self->test_setBillingAddress();
    $self->test_removeBillingAddress();
    $self->test_setShippingAddress();
    $self->test_removeShippingAddress();
    done_testing();
}



sub test_patch_modifyProductLineItem {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $Resource = DE_EPAGES::Order::API::Resource::Cart->new();

    my $hPatchRequest = [
        {
            'op' => 'add',
            'path' => '/lineItemContainer/productLineItems/53AD3774-B3DC-155B-3E97-AC1432AA8AC2',
            'value' => {
                'quantity' => '11',
            },
        }
    ];

    my $Basket = $Shop->path('AnonymousUsers/cartTester01/Baskets/cartTest01');
    my $hBasketData = $Resource->patch($Basket, $hPatchRequest);

    # check basket
    my $LineItemContainer = $Basket->container();
    my $aLineItems = $LineItemContainer->children();
    is(scalar @{$aLineItems}, 4,"$SubTestName - test if there are still 4 different lineitems after the patch");
    for my $LineItem (@$aLineItems) {
        if($LineItem->class()->existsAttribute('Product')) {
            if($LineItem->get('Product')->guid() eq '53AD3774-B3DC-155B-3E97-AC1432AA8AC2') {
                is($LineItem->get('Quantity'), 11,"$SubTestName - test if amount of lineitem was set correctly."); ## no critic (ProhibitMagicNumbers)
            }
        }
    }

    # check returned data
    my $aProductLineItems = $hBasketData->{'lineItemContainer'}->{'productLineItems'};
    is(scalar @{$aProductLineItems}, 1,"$SubTestName - test if there is still only one product lineitem after the patch");
    for my $hProductLineItem (@$aProductLineItems) {
        if($hProductLineItem->{'productId'} eq '53AD3774-B3DC-155B-3E97-AC1432AA8AC2') {
            is($hProductLineItem->{'quantity'}->{'amount'}, 11,"$SubTestName - test if amount of lineitem was set correctly."); ## no critic (ProhibitMagicNumbers)
        }
    }

    return;
}



sub test_patch_addProductLineItem {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $Resource = DE_EPAGES::Order::API::Resource::Cart->new();

    my $hPatchRequest = [
        {
            'op' => 'add',
            'path' => '/lineItemContainer/productLineItems',
            'value' => {
                'productId' => '53AA962B-1F09-B778-E3AF-AC1432AA7986',
                'quantity' => '3',
            },
        }
    ];

    my $Basket = $Shop->path('AnonymousUsers/cartTester01/Baskets/cartTest01');
    my $hBasketData = $Resource->patch($Basket, $hPatchRequest);

    # check basket
    my $LineItemContainer = $Basket->container();
    my $aLineItems = $LineItemContainer->children();
    is(scalar @{$aLineItems}, 5,"$SubTestName - test if there are 5 different lineitems after the patch");
    for my $LineItem (@$aLineItems) {
        if($LineItem->class()->existsAttribute('Product')) {
            if($LineItem->get('Product')->guid() eq '53AA962B-1F09-B778-E3AF-AC1432AA7986') {
                $GuidOfAddedLineItem = $LineItem->guid();
                is($LineItem->get('Quantity'), 3,"$SubTestName - test if amount of lineitem was set correctly.");
            }
        }
    }

    # check returned data
    my $aProductLineItems = $hBasketData->{'lineItemContainer'}->{'productLineItems'};
    is(scalar @{$aProductLineItems}, 2,"$SubTestName - test if there are 2 different product lineitems after the patch");
    for my $hProductLineItem (@$aProductLineItems) {
        if($hProductLineItem->{'productId'} eq '53AA962B-1F09-B778-E3AF-AC1432AA7986') {
            is($hProductLineItem->{'quantity'}->{'amount'}, 3,"$SubTestName - test if amount of lineitem was set correctly.");
        }
    }

    return;
}



sub test_patch_removeProductLineItem {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $Resource = DE_EPAGES::Order::API::Resource::Cart->new();

    my $hPatchRequest = [
        {
            'op' => 'remove',
            'path' => '/lineItemContainer/productLineItems/'.$GuidOfAddedLineItem,
        }
    ];

    my $Basket = $Shop->path('AnonymousUsers/cartTester01/Baskets/cartTest01');
    my $hBasketData = $Resource->patch($Basket, $hPatchRequest);

    # check basket
    my $LineItemContainer = $Basket->container();
    my $aLineItems = $LineItemContainer->children();
    is(scalar @{$aLineItems}, 4,"$SubTestName - test if there are still 4 different lineitems after the patch");
    my $DeletetionSuccessful = 1;
    for my $LineItem (@$aLineItems) {
        if($LineItem->class()->existsAttribute('Product')) {
            if($LineItem->get('Product')->guid() eq '53AD3774-B3DC-155B-3E97-AC1432AA8AC2') {
                $DeletetionSuccessful = 0;
            }
        }
    }
    ok($DeletetionSuccessful, "$SubTestName - test if deletion was successful.");

    # check returned data
    my $aProductLineItems = $hBasketData->{'lineItemContainer'}->{'productLineItems'};
    is(scalar @{$aProductLineItems}, 1,"$SubTestName - test if there is still only one product lineitem after the patch");
    my $DeletedLineItemNotInDataAnymore = 1;
    for my $hProductLineItem (@$aProductLineItems) {
        if($hProductLineItem->{'productId'} eq '53AD3774-B3DC-155B-3E97-AC1432AA8AC2') {
            $DeletedLineItemNotInDataAnymore = 0;
        }
    }
    ok($DeletedLineItemNotInDataAnymore, "$SubTestName - test deleted lineitem does not appear in return data anymore.");

    return;
}



sub test_setBillingAddress {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $Resource = DE_EPAGES::Order::API::Resource::Cart->new();

    my $hPatchRequest = [
        {
            'op' => 'add',
            'path' => '/billingAddress',
            'value' => {
                'birthday' => '1903-02-01',
                'city' => 'Hamburg',
                'company' => 'epages',
                'country' => 'DE',
                'emailAddress' => 'email@test.org',
                'firstName' => 'John',
                'lastName' => 'Doe',
                'salutation' => 'Salve!',
                'state' => 'HH',
                'street' => 'Pilatuspool 2',
                'streetDetails' => '2nd Backyard',
                'title' => 'Dr.',
                'vatId' => '123-456-7-890',
                'zipCode' => '20355',
            },
        }
    ];

    my $Basket = $Shop->path('AnonymousUsers/cartTester01/Baskets/cartTest01');
    my $hBasketData = $Resource->patch($Basket, $hPatchRequest);

    # check basket
    my $BillingAddress = $Basket->get('BillingAddress');
    is($BillingAddress->get('Birthday'), '1903-02-01T00:00:00', "$SubTestName - test if birthday of billing address is set correctly.");
    is($BillingAddress->get('City'), 'Hamburg', "$SubTestName - test if city of billing address is set correctly.");
    is($BillingAddress->get('Company'), 'epages', "$SubTestName - test if company of billing address is set correctly.");
    is($BillingAddress->get('CountryID'), '276', "$SubTestName - test if countryID of billing address is set correctly.");
    is($BillingAddress->get('EMail'), 'email@test.org', "$SubTestName - test if email of billing address is set correctly.");
    is($BillingAddress->get('FirstName'), 'John', "$SubTestName - test if firstname of billing address is set correctly.");
    is($BillingAddress->get('LastName'), 'Doe', "$SubTestName - test if lastname of billing address is set correctly.");
    is($BillingAddress->get('Salutation'), 'Salve!', "$SubTestName - test if salutation of billing address is set correctly.");
    is($BillingAddress->get('State'), 'HH', "$SubTestName - test if state of billing address is set correctly.");
    is($BillingAddress->get('Street'), 'Pilatuspool 2', "$SubTestName - test if street of billing address is set correctly.");
    is($BillingAddress->get('Street2'), '2nd Backyard', "$SubTestName - test if street details of billing address is set correctly.");
    is($BillingAddress->get('Title'), 'Dr.', "$SubTestName - test if title of billing address is set correctly.");
    is($BillingAddress->get('VATID'), '123-456-7-890', "$SubTestName - test if vat id of billing address is set correctly.");
    is($BillingAddress->get('Zipcode'), '20355', "$SubTestName - test if ZipCode of billing address is set correctly.");

    # check returned data
    my $hBillingAddress = $hBasketData->{'billingAddress'};
    is_deeply($hBillingAddress, $self->_createExpectedAddress(), "$SubTestName - test if correct billing address was returned");

    return;
}



sub test_removeBillingAddress {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $Resource = DE_EPAGES::Order::API::Resource::Cart->new();

    my $hPatchRequest = [
        {
            'op' => 'remove',
            'path' => '/billingAddress',
        }
    ];

    my $Basket = $Shop->path('AnonymousUsers/cartTester01/Baskets/cartTest01');
    my $hBasketData = $Resource->patch($Basket, $hPatchRequest);

    # check basket
    my $BillingAddress = $Basket->get('BillingAddress');
    is($BillingAddress, undef, "$SubTestName - test if basket does not has a billing address.");

    # check returned data
    my $hBillingAddress = $hBasketData->{'billingAddress'};
    is($hBillingAddress, undef, "$SubTestName - test if response basket data does not contain a billing address.");

    return;
}



sub test_setShippingAddress {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $Resource = DE_EPAGES::Order::API::Resource::Cart->new();

    my $hPatchRequest = [
        {
            'op' => 'add',
            'path' => '/shippingAddress',
            'value' => {
                'birthday' => '1903-02-01',
                'city' => 'Hamburg',
                'company' => 'epages',
                'country' => 'DE',
                'emailAddress' => 'email@test.org',
                'firstName' => 'John',
                'lastName' => 'Doe',
                'salutation' => 'Salve!',
                'state' => 'HH',
                'street' => 'Pilatuspool 2',
                'streetDetails' => '2nd Backyard',
                'title' => 'Dr.',
                'vatId' => '123-456-7-890',
                'zipCode' => '20355',
            },
        }
    ];

    my $Basket = $Shop->path('AnonymousUsers/cartTester01/Baskets/cartTest01');
    my $hBasketData = $Resource->patch($Basket, $hPatchRequest);

    # check basket
    my $ShippingAddress = $Basket->get('ShippingAddress');
    is($ShippingAddress->get('Birthday'), '1903-02-01T00:00:00', "$SubTestName - test if birthday of shipping address is set correctly.");
    is($ShippingAddress->get('City'), 'Hamburg', "$SubTestName - test if city of shipping address is set correctly.");
    is($ShippingAddress->get('Company'), 'epages', "$SubTestName - test if company of shipping address is set correctly.");
    is($ShippingAddress->get('CountryID'), '276', "$SubTestName - test if countryID of shipping address is set correctly.");
    is($ShippingAddress->get('EMail'), 'email@test.org', "$SubTestName - test if email of shipping address is set correctly.");
    is($ShippingAddress->get('FirstName'), 'John', "$SubTestName - test if firstname of shipping address is set correctly.");
    is($ShippingAddress->get('LastName'), 'Doe', "$SubTestName - test if lastname of shipping address is set correctly.");
    is($ShippingAddress->get('Salutation'), 'Salve!', "$SubTestName - test if salutation of shipping address is set correctly.");
    is($ShippingAddress->get('State'), 'HH', "$SubTestName - test if state of shipping address is set correctly.");
    is($ShippingAddress->get('Street'), 'Pilatuspool 2', "$SubTestName - test if street of shipping address is set correctly.");
    is($ShippingAddress->get('Street2'), '2nd Backyard', "$SubTestName - test if street details of shipping address is set correctly.");
    is($ShippingAddress->get('Title'), 'Dr.', "$SubTestName - test if title of shipping address is set correctly.");
    is($ShippingAddress->get('VATID'), '123-456-7-890', "$SubTestName - test if vat id of shipping address is set correctly.");
    is($ShippingAddress->get('Zipcode'), '20355', "$SubTestName - test if ZipCode of shipping address is set correctly.");

    # check returned data
    my $hShippingAddress = $hBasketData->{'shippingAddress'};
    is_deeply($hShippingAddress, $self->_createExpectedAddress(), "$SubTestName - test if correct shipping address was returned");

    return;
}



sub test_removeShippingAddress {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $Resource = DE_EPAGES::Order::API::Resource::Cart->new();

    my $hPatchRequest = [
        {
            'op' => 'remove',
            'path' => '/shippingAddress',
        }
    ];

    my $Basket = $Shop->path('AnonymousUsers/cartTester01/Baskets/cartTest01');
    my $hBasketData = $Resource->patch($Basket, $hPatchRequest);

    # check basket
    my $ShippingAddress = $Basket->get('ShippingAddress');
    is($ShippingAddress, undef, "$SubTestName - test if basket does not has a shipping address.");

    # check returned data
    my $hShippingAddress = $hBasketData->{'shippingAddress'};
    is($hShippingAddress, undef, "$SubTestName - test if response basket data does not contain a shipping address.");

    return;
}



sub _createExpectedAddress {
    return {
        'birthday' => '1903-02-01',
        'city' => 'Hamburg',
        'company' => 'epages',
        'country' => 'DE',
        'emailAddress' => 'email@test.org',
        'firstName' => 'John',
        'lastName' => 'Doe',
        'salutation' => 'Salve!',
        'state' => 'HH',
        'street' => 'Pilatuspool 2',
        'streetDetails' => '2nd Backyard',
        'title' => 'Dr.',
        'vatId' => '123-456-7-890',
        'zipCode' => '20355',
      };
}



__PACKAGE__->runScript();

1;